unit WinLayerControls;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Forms, Controls,
  ExtCtrls, PngImage, WinLayerCore, WinLayerConts;

type
  TWinLayerButton = class(TWinLayerCustomControl)
  private
    FButtonState: TWinLayerFormButtonState;

    procedure SetButtonState(Value: TWinLayerFormButtonState);
    procedure CMTEXTCHANGED(var Message: TMessage); message CM_TEXTCHANGED;
    procedure CMMOUSELEAVE(var Message: TMessage); message CM_MOUSELEAVE;
    procedure CMENABLEDCHANGED(var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure WMMOUSEMOVE(var Message: TWMMOUSEMOVE); message WM_MOUSEMOVE;
    procedure WMLBUTTONDOWN(var Message: TWMLBUTTONDOWN); message WM_LBUTTONDOWN;
    procedure WMLBUTTONUP(var Message: TWMLBUTTONUP); message WM_LBUTTONUP;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function GetDIB: TDIBSection; override;
  published
    property Align;
    property Text;
    property Font;
    property Alignment;
    property OnClick;
    property OnContextPopup;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDock;
    property OnStartDrag;
  end;

  TWinLayerCheckBox = class(TWinLayerButton)
  private
    FChecked: Boolean;

    procedure SetChecked(Value: Boolean);
    procedure WMLBUTTONUP(var Message: TWMLBUTTONUP); message WM_LBUTTONUP;
  protected
    procedure SetAlignment(Value: TAlignment); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function GetDIB: TDIBSection; override;
  published
    property Checked: Boolean read FChecked write SetChecked default False;
  end;

  TWinLayerRadioBox = class(TWinLayerCheckBox)
  private
    FChecked: Boolean;

    procedure SetChecked(Value: Boolean);
    procedure WMLBUTTONUP(var Message: TWMLBUTTONUP); message WM_LBUTTONUP;
  protected
    procedure SetAlignment(Value: TAlignment); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function GetDIB: TDIBSection; override;
  published
    property Checked: Boolean read FChecked write SetChecked default False;
  end;

  TWinLayerProgressBarOrientation = (pbHorizontal, pbVertical);
  TWinLayerProgressBar = class(TWinLayerCustomControl)
  private
    FMin: Integer;
    FMax: Integer;
    FPosition: Integer;
    FOrientation: TWinLayerProgressBarOrientation;
    FOnProgress: TNotifyEvent;

    procedure SetMax(Value: Integer);
    procedure SetMin(Value: Integer);
    procedure SetPosition(Value: Integer);
    procedure SetOrientation(Value: TWinLayerProgressBarOrientation);

    procedure CMTEXTCHANGED(var Message: TMessage); message CM_TEXTCHANGED;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    function GetDIB: TDIBSection; override;
  published
    property Align;
    property Text;
    property Font;
    property Alignment;
    property Max: Integer read FMax write SetMax default 100;
    property Min: Integer read FMin write SetMin default 0;
    property Position: Integer read FPosition write SetPosition default 0;
    property Orientation: TWinLayerProgressBarOrientation read FOrientation write SetOrientation;
    property OnProgress: TNotifyEvent read FOnProgress write FOnProgress;
    property OnClick;
    property OnContextPopup;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDock;
    property OnStartDrag;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('WinLayerLib', [TWinLayerProgressBar, TWinLayerButton,
    TWinLayerCheckBox, TWinLayerRadioBox]);
end;

const
  TextFormat: array[TAlignment] of DWORD = (DT_LEFT, DT_RIGHT, DT_CENTER);

{ TWinLayerProgressBar }

constructor TWinLayerProgressBar.Create(AOwner: TComponent);
begin
  inherited;
  FMax := 100;
  FMin := 0;
  FPosition := 0;
end;

destructor TWinLayerProgressBar.Destroy;
begin
  inherited;
end;

procedure TWinLayerProgressBar.CMTEXTCHANGED(var Message: TMessage);
begin
  inherited;
  Draw;
end;

procedure TWinLayerProgressBar.SetMax(Value: Integer);
begin
  if (Value <> FMax) then
  begin
    FMax := Value;
    Draw;
  end;
end;

procedure TWinLayerProgressBar.SetMin(Value: Integer);
begin
  if (Value <> FMin) then
  begin
    FMin := Value;
    Draw;
  end;
end;

procedure TWinLayerProgressBar.SetPosition(Value: Integer);
begin
  if (Value <> FPosition) and (Value >= FMin) and (Value <= FMax) then
  begin
    FPosition := Value;
    Draw;
    if Assigned(FOnProgress) then
      FOnProgress(Self);
  end;
end;

procedure TWinLayerProgressBar.SetOrientation(Value: TWinLayerProgressBarOrientation);
begin
end;

function TWinLayerProgressBar.GetDIB: TDIBSection;
var
  DIB, Tmp: TDIBSection;
  r: TRect;
begin
  DIB := TDIBSection.Create;
  DIB.SetSize(Width, Height, 32);
  DIB.DrawDIB(DIB.Rect, WinLayerForm.SurfaceWindow, BoundsRect);

  with WinLayerForm do
  begin
    DIB.DrawPNG(Rect(0, 0,
      WLibrary.Items[WinLayer_src_horizontal_progressbar_bckg_lefttop].Int(2),
      WLibrary.Items[WinLayer_src_horizontal_progressbar_bckg_lefttop].Int(3)),
      WLibrary.Items[WinLayer_src_image].AsImage,
      WLibrary.Items[WinLayer_src_horizontal_progressbar_bckg_lefttop].Rect, False);

    DIB.DrawPNG(Rect(0,
      WLibrary.Items[WinLayer_src_horizontal_progressbar_bckg_lefttop].Int(3),
      WLibrary.Items[WinLayer_src_horizontal_progressbar_bckg_left].Int(2),
      DIB.Height - WLibrary.Items[WinLayer_src_horizontal_progressbar_bckg_leftbottom].Int(3)),
      WLibrary.Items[WinLayer_src_image].AsImage,
      WLibrary.Items[WinLayer_src_horizontal_progressbar_bckg_left].Rect,
      False);

    DIB.DrawPNG(Rect(0,
      DIB.Height - WLibrary.Items[WinLayer_src_horizontal_progressbar_bckg_leftbottom].Int(3),
      WLibrary.Items[WinLayer_src_horizontal_progressbar_bckg_leftbottom].Int(2),
      DIB.Height),
      WLibrary.Items[WinLayer_src_image].AsImage,
      WLibrary.Items[WinLayer_src_horizontal_progressbar_bckg_leftbottom].Rect, False);

    DIB.DrawPNG(Rect(WLibrary.Items[WinLayer_src_horizontal_progressbar_bckg_lefttop].Int(2),
      0,
      DIB.Width - WLibrary.Items[WinLayer_src_horizontal_progressbar_bckg_righttop].Int(2),
      WLibrary.Items[WinLayer_src_horizontal_progressbar_bckg_middletop].Int(3)),
      WLibrary.Items[WinLayer_src_image].AsImage,
      WLibrary.Items[WinLayer_src_horizontal_progressbar_bckg_middletop].Rect, False);
                               
    DIB.DrawPNG(Rect(WLibrary.Items[WinLayer_src_horizontal_progressbar_bckg_left].Int(2),
      WLibrary.Items[WinLayer_src_horizontal_progressbar_bckg_middletop].Int(3),
      DIB.Width - WLibrary.Items[WinLayer_src_horizontal_progressbar_bckg_right].Int(2),
      DIB.Height - WLibrary.Items[WinLayer_src_horizontal_progressbar_bckg_middlebottom].Int(3)),
      WLibrary.Items[WinLayer_src_image].AsImage,
      WLibrary.Items[WinLayer_src_horizontal_progressbar_bckg_middle].Rect, False);

    DIB.DrawPNG(Rect(WLibrary.Items[WinLayer_src_horizontal_progressbar_bckg_leftbottom].Int(2),
      DIB.Height - WLibrary.Items[WinLayer_src_horizontal_progressbar_bckg_middlebottom].Int(3),
      DIB.Width - WLibrary.Items[WinLayer_src_horizontal_progressbar_bckg_rightbottom].Int(2),
      DIB.Height),
      WLibrary.Items[WinLayer_src_image].AsImage,
      WLibrary.Items[WinLayer_src_horizontal_progressbar_bckg_middlebottom].Rect, False);

    DIB.DrawPNG(Rect(DIB.Width - WLibrary.Items[WinLayer_src_horizontal_progressbar_bckg_righttop].Int(2),
      0, DIB.Width,
      WLibrary.Items[WinLayer_src_horizontal_progressbar_bckg_righttop].Int(3)),
      WLibrary.Items[WinLayer_src_image].AsImage,
      WLibrary.Items[WinLayer_src_horizontal_progressbar_bckg_righttop].Rect, False);

    DIB.DrawPNG(Rect(DIB.Width - WLibrary.Items[WinLayer_src_horizontal_progressbar_bckg_right].Int(2),
      WLibrary.Items[WinLayer_src_horizontal_progressbar_bckg_righttop].Int(3), DIB.Width,
      DIB.Height - WLibrary.Items[WinLayer_src_horizontal_progressbar_bckg_rightbottom].Int(3)),
      WLibrary.Items[WinLayer_src_image].AsImage,
      WLibrary.Items[WinLayer_src_horizontal_progressbar_bckg_right].Rect, False);

    DIB.DrawPNG(Rect(DIB.Width - WLibrary.Items[WinLayer_src_horizontal_progressbar_bckg_rightbottom].Int(2),
      DIB.Height - WLibrary.Items[WinLayer_src_horizontal_progressbar_bckg_rightbottom].Int(3),
      DIB.Width, DIB.Height),
      WLibrary.Items[WinLayer_src_image].AsImage,
      WLibrary.Items[WinLayer_src_horizontal_progressbar_bckg_rightbottom].Rect, False);
             
    Tmp := TDIBSection.Create;
    Tmp.Assign(DIB);

    Tmp.DrawPNG(Rect(0, 0,
      WLibrary.Items[WinLayer_src_horizontal_progressbar_fill_lefttop].Int(2),
      WLibrary.Items[WinLayer_src_horizontal_progressbar_fill_lefttop].Int(3)),
      WLibrary.Items[WinLayer_src_image].AsImage,
      WLibrary.Items[WinLayer_src_horizontal_progressbar_fill_lefttop].Rect, False);

    Tmp.DrawPNG(Rect(0,
      WLibrary.Items[WinLayer_src_horizontal_progressbar_fill_lefttop].Int(3),
      WLibrary.Items[WinLayer_src_horizontal_progressbar_fill_left].Int(2),
      Tmp.Height - WLibrary.Items[WinLayer_src_horizontal_progressbar_fill_leftbottom].Int(3)),
      WLibrary.Items[WinLayer_src_image].AsImage,
      WLibrary.Items[WinLayer_src_horizontal_progressbar_fill_left].Rect,
      False);

    Tmp.DrawPNG(Rect(0,
      Tmp.Height - WLibrary.Items[WinLayer_src_horizontal_progressbar_fill_leftbottom].Int(3),
      WLibrary.Items[WinLayer_src_horizontal_progressbar_fill_leftbottom].Int(2),
      Tmp.Height),
      WLibrary.Items[WinLayer_src_image].AsImage,
      WLibrary.Items[WinLayer_src_horizontal_progressbar_fill_leftbottom].Rect, False);

    Tmp.DrawPNG(Rect(WLibrary.Items[WinLayer_src_horizontal_progressbar_fill_lefttop].Int(2),
      0,
      Tmp.Width - WLibrary.Items[WinLayer_src_horizontal_progressbar_fill_righttop].Int(2),
      WLibrary.Items[WinLayer_src_horizontal_progressbar_fill_middletop].Int(3)),
      WLibrary.Items[WinLayer_src_image].AsImage,
      WLibrary.Items[WinLayer_src_horizontal_progressbar_fill_middletop].Rect, False);
                               
    Tmp.DrawPNG(Rect(WLibrary.Items[WinLayer_src_horizontal_progressbar_fill_left].Int(2),
      WLibrary.Items[WinLayer_src_horizontal_progressbar_fill_middletop].Int(3),
      Tmp.Width - WLibrary.Items[WinLayer_src_horizontal_progressbar_fill_right].Int(2),
      Tmp.Height - WLibrary.Items[WinLayer_src_horizontal_progressbar_fill_middlebottom].Int(3)),
      WLibrary.Items[WinLayer_src_image].AsImage,
      WLibrary.Items[WinLayer_src_horizontal_progressbar_fill_middle].Rect, False);

    Tmp.DrawPNG(Rect(WLibrary.Items[WinLayer_src_horizontal_progressbar_fill_leftbottom].Int(2),
      Tmp.Height - WLibrary.Items[WinLayer_src_horizontal_progressbar_fill_middlebottom].Int(3),
      Tmp.Width - WLibrary.Items[WinLayer_src_horizontal_progressbar_fill_rightbottom].Int(2),
      Tmp.Height),
      WLibrary.Items[WinLayer_src_image].AsImage,
      WLibrary.Items[WinLayer_src_horizontal_progressbar_fill_middlebottom].Rect, False);

    Tmp.DrawPNG(Rect(Tmp.Width - WLibrary.Items[WinLayer_src_horizontal_progressbar_fill_righttop].Int(2),
      0, Tmp.Width,
      WLibrary.Items[WinLayer_src_horizontal_progressbar_fill_righttop].Int(3)),
      WLibrary.Items[WinLayer_src_image].AsImage,
      WLibrary.Items[WinLayer_src_horizontal_progressbar_fill_righttop].Rect, False);

    Tmp.DrawPNG(Rect(Tmp.Width - WLibrary.Items[WinLayer_src_horizontal_progressbar_fill_right].Int(2),
      WLibrary.Items[WinLayer_src_horizontal_progressbar_fill_righttop].Int(3), Tmp.Width,
      Tmp.Height - WLibrary.Items[WinLayer_src_horizontal_progressbar_fill_rightbottom].Int(3)),
      WLibrary.Items[WinLayer_src_image].AsImage,
      WLibrary.Items[WinLayer_src_horizontal_progressbar_fill_right].Rect, False);

    Tmp.DrawPNG(Rect(Tmp.Width - WLibrary.Items[WinLayer_src_horizontal_progressbar_fill_rightbottom].Int(2),
      Tmp.Height - WLibrary.Items[WinLayer_src_horizontal_progressbar_fill_rightbottom].Int(3),
      Tmp.Width, Tmp.Height),
      WLibrary.Items[WinLayer_src_image].AsImage,
      WLibrary.Items[WinLayer_src_horizontal_progressbar_fill_rightbottom].Rect, False);

    r := Rect(0, 0, Round(Tmp.Width * ((FPosition - FMin) / (FMax - FMin))), Tmp.Height);
    DIB.DrawDIB(r, Tmp, r);

    Tmp.Free;   
  end;

  DIB.Canvas.Font.Assign(Font);
  DIB.Canvas.Brush.Style := bsClear;
  r := DIB.Rect;
  DrawText(DIB.Canvas.Handle, PCHAR(Text), Length(Text), r,
    DT_VCENTER or DT_SINGLELINE or DT_WORDBREAK or DT_WORD_ELLIPSIS or TextFormat[Alignment]);
  DIB.Canvas.Brush.Style := bsSolid;
  DIB.PremultiplyRect(DIB.Rect);

  Result := DIB;
end;

{ TWinLayerButton }

constructor TWinLayerButton.Create(AOwner: TComponent);
begin
  inherited;
  FButtonState := bsNormal;
end;

destructor TWinLayerButton.Destroy;
begin
  inherited;
end;

procedure TWinLayerButton.CMTEXTCHANGED(var Message: TMessage);
begin
  inherited;
  Draw;
end;

function TWinLayerButton.GetDIB: TDIBSection;
const
  ButtonState: array[TWinLayerFormButtonState, 0..8] of String =
  (
  (WinLayer_src_button_normal_lefttop, WinLayer_src_button_normal_left,
  WinLayer_src_button_normal_leftbottom, WinLayer_src_button_normal_middletop,
  WinLayer_src_button_normal_middle, WinLayer_src_button_normal_middlebottom,
  WinLayer_src_button_normal_righttop, WinLayer_src_button_normal_right,
  WinLayer_src_button_normal_rightbottom),

  (WinLayer_src_button_hover_lefttop, WinLayer_src_button_hover_left,
  WinLayer_src_button_hover_leftbottom, WinLayer_src_button_hover_middletop,
  WinLayer_src_button_hover_middle, WinLayer_src_button_hover_middlebottom,
  WinLayer_src_button_hover_righttop, WinLayer_src_button_hover_right,
  WinLayer_src_button_hover_rightbottom),

  (WinLayer_src_button_push_lefttop, WinLayer_src_button_push_left,
  WinLayer_src_button_push_leftbottom, WinLayer_src_button_push_middletop,
  WinLayer_src_button_push_middle, WinLayer_src_button_push_middlebottom,
  WinLayer_src_button_push_righttop, WinLayer_src_button_push_right,
  WinLayer_src_button_push_rightbottom),

  (WinLayer_src_button_disable_lefttop, WinLayer_src_button_disable_left,
  WinLayer_src_button_disable_leftbottom, WinLayer_src_button_disable_middletop,
  WinLayer_src_button_disable_middle, WinLayer_src_button_disable_middlebottom,
  WinLayer_src_button_disable_righttop, WinLayer_src_button_disable_right,
  WinLayer_src_button_disable_rightbottom)
  );
var
  DIB: TDIBSection;
  r: TRect;
begin
  DIB := TDIBSection.Create;
  DIB.SetSize(Width, Height, 32);
  DIB.DrawDIB(DIB.Rect, WinLayerForm.SurfaceWindow, BoundsRect);

  with WinLayerForm do
  begin
    DIB.DrawPNG(Rect(0, 0,
      WLibrary.Items[ButtonState[FButtonState, 0]].Int(2),
      WLibrary.Items[ButtonState[FButtonState, 0]].Int(3)),
      WLibrary.Items[WinLayer_src_image].AsImage,
      WLibrary.Items[ButtonState[FButtonState, 0]].Rect, False);

    DIB.DrawPNG(Rect(0,
      WLibrary.Items[ButtonState[FButtonState, 0]].Int(3),
      WLibrary.Items[ButtonState[FButtonState, 1]].Int(2),
      DIB.Height - WLibrary.Items[ButtonState[FButtonState, 2]].Int(3)),
      WLibrary.Items[WinLayer_src_image].AsImage,
      WLibrary.Items[ButtonState[FButtonState, 1]].Rect,
      False);

    DIB.DrawPNG(Rect(0,
      DIB.Height - WLibrary.Items[ButtonState[FButtonState, 2]].Int(3),
      WLibrary.Items[ButtonState[FButtonState, 2]].Int(2),
      DIB.Height),
      WLibrary.Items[WinLayer_src_image].AsImage,
      WLibrary.Items[ButtonState[FButtonState, 2]].Rect, False);

    DIB.DrawPNG(Rect(WLibrary.Items[ButtonState[FButtonState, 0]].Int(2),
      0,
      DIB.Width - WLibrary.Items[ButtonState[FButtonState, 6]].Int(2),
      WLibrary.Items[ButtonState[FButtonState, 3]].Int(3)),
      WLibrary.Items[WinLayer_src_image].AsImage,
      WLibrary.Items[ButtonState[FButtonState, 3]].Rect, False);
                               
    DIB.DrawPNG(Rect(WLibrary.Items[ButtonState[FButtonState, 1]].Int(2),
      WLibrary.Items[ButtonState[FButtonState, 3]].Int(3),
      DIB.Width - WLibrary.Items[ButtonState[FButtonState, 7]].Int(2),
      DIB.Height - WLibrary.Items[ButtonState[FButtonState, 5]].Int(3)),
      WLibrary.Items[WinLayer_src_image].AsImage,
      WLibrary.Items[ButtonState[FButtonState, 4]].Rect, False);

    DIB.DrawPNG(Rect(WLibrary.Items[ButtonState[FButtonState, 2]].Int(2),
      DIB.Height - WLibrary.Items[ButtonState[FButtonState, 5]].Int(3),
      DIB.Width - WLibrary.Items[ButtonState[FButtonState, 8]].Int(2),
      DIB.Height),
      WLibrary.Items[WinLayer_src_image].AsImage,
      WLibrary.Items[ButtonState[FButtonState, 5]].Rect, False);
                            
    DIB.DrawPNG(Rect(DIB.Width - WLibrary.Items[ButtonState[FButtonState, 6]].Int(2),
      0, DIB.Width,
      WLibrary.Items[ButtonState[FButtonState, 6]].Int(3)),
      WLibrary.Items[WinLayer_src_image].AsImage,
      WLibrary.Items[ButtonState[FButtonState, 6]].Rect, False);

    DIB.DrawPNG(Rect(DIB.Width - WLibrary.Items[ButtonState[FButtonState, 7]].Int(2),
      WLibrary.Items[ButtonState[FButtonState, 6]].Int(3), DIB.Width,
      DIB.Height - WLibrary.Items[ButtonState[FButtonState, 8]].Int(3)),
      WLibrary.Items[WinLayer_src_image].AsImage,
      WLibrary.Items[ButtonState[FButtonState, 7]].Rect, False);

    DIB.DrawPNG(Rect(DIB.Width - WLibrary.Items[ButtonState[FButtonState, 8]].Int(2),
      DIB.Height - WLibrary.Items[ButtonState[FButtonState, 8]].Int(3),
      DIB.Width, DIB.Height),
      WLibrary.Items[WinLayer_src_image].AsImage,
      WLibrary.Items[ButtonState[FButtonState, 8]].Rect, False);
  end;

  DIB.Canvas.Font.Assign(Font);
  DIB.Canvas.Brush.Style := bsClear;
  r := DIB.Rect;
  DrawText(DIB.Canvas.Handle, PCHAR(Text), Length(Text), r,
    DT_VCENTER or DT_SINGLELINE or DT_WORDBREAK or DT_WORD_ELLIPSIS or TextFormat[Alignment]);
  DIB.Canvas.Brush.Style := bsSolid;

  DIB.PremultiplyRect(DIB.Rect);

  Result := DIB;
end;

procedure TWinLayerButton.SetButtonState(Value: TWinLayerFormButtonState);
begin
  if (Value <> FButtonState) then
  begin
    FButtonState := Value;
    Draw;
  end;
end;

procedure TWinLayerButton.CMMOUSELEAVE(var Message: TMessage);
begin
  inherited;
  if (FButtonState <> bsDisable) then
    SetButtonState(bsNormal);
end;

procedure TWinLayerButton.CMENABLEDCHANGED(var Message: TMessage);
begin
  inherited;
  if not Enabled then
    SetButtonState(bsDisable) else
  if PtInRect(ClientRect, Mouse.CursorPos) then
    SetButtonState(bsHover) else
    SetButtonState(bsNormal);
end;

procedure TWinLayerButton.WMLBUTTONDOWN(var Message: TWMLBUTTONDOWN);
begin
  inherited;
  if (FButtonState <> bsDisable) then
  begin
    SetFocus;
    SetButtonState(bsPush);
  end;
end;

procedure TWinLayerButton.WMLBUTTONUP(var Message: TWMLBUTTONUP);
begin
  inherited;
  if (FButtonState <> bsDisable) then
  if PtInRect(ClientRect, SmallPointToPoint(Message.Pos)) then
    SetButtonState(bsHover) else
    SetButtonState(bsNormal);
end;

procedure TWinLayerButton.WMMOUSEMOVE(var Message: TWMMOUSEMOVE);
begin
  inherited;
  if (FButtonState <> bsDisable) then
  begin
    if PtInRect(ClientRect, SmallPointToPoint(Message.Pos)) then
    begin
      if (csLButtonDown in ControlState) then
      SetButtonState(bsPush) else
      SetButtonState(bsHover);
    end else
      SetButtonState(bsNormal);
  end;
end;

{ TWinLayerCheckBox }

constructor TWinLayerCheckBox.Create(AOwner: TComponent);
begin
  inherited;
  FChecked := False;
  FButtonState := bsNormal;
end;

destructor TWinLayerCheckBox.Destroy;
begin
  inherited;
end;

procedure TWinLayerCheckBox.SetChecked(Value: Boolean);
begin
  if (Value <> FChecked) then
  begin
    FChecked := Value;
    Draw;
  end;
end;

procedure TWinLayerCheckBox.SetAlignment(Value: TAlignment);
begin
  if (Value <> taCenter) then
    inherited;
end;

procedure TWinLayerCheckBox.WMLBUTTONUP(var Message: TWMLBUTTONUP);
begin
  inherited;
  if (FButtonState <> bsDisable) and PtInRect(ClientRect, SmallPointToPoint(Message.Pos)) then
    SetChecked(not FChecked);
end;

function TWinLayerCheckBox.GetDIB: TDIBSection;
const
  ButtonState: array[TWinLayerFormButtonState, Boolean] of String =
  ((WinLayer_src_checkbox_unchecked_normal, WinLayer_src_checkbox_checked_normal),
  (WinLayer_src_checkbox_unchecked_hover, WinLayer_src_checkbox_checked_hover),
  (WinLayer_src_checkbox_unchecked_push, WinLayer_src_checkbox_checked_push),
  (WinLayer_src_checkbox_unchecked_disable, WinLayer_src_checkbox_checked_disable));
var
  DIB: TDIBSection;
  r, r2, r3: TRect;
begin
  DIB := TDIBSection.Create;
  DIB.SetSize(Width, Height, 32);
  DIB.DrawDIB(DIB.Rect, WinLayerForm.SurfaceWindow, BoundsRect);
  DIB.Canvas.Font.Assign(Font);

  with WinLayerForm do
  begin
    r := WLibrary.Items[ButtonState[FButtonState, FChecked]].Rect;

    r2.Top := (DIB.Height - r.Bottom + r.Top) div 2;
    r2.Bottom := r2.Top + r.Bottom - r.Top;
    r3.Top := (DIB.Height - DIB.Canvas.TextHeight('W')) div 2;
    r3.Bottom := r3.Top + DIB.Canvas.TextHeight('W');
    
    case Alignment of
      taLeftJustify:
      begin
        r2.Left := 0;
        r2.Right := r2.Left + r.Right - r.Left;
        r3.Left := r2.Right;
        r3.Right := DIB.Width;
      end;
      taRightJustify:
      begin
        r2.Left := DIB.Width - r.Right + r.Left;
        r2.Right := DIB.Width;
        r3.Left := 0;
        r3.Right := r2.Left;
      end;
    end;

    DIB.DrawPNG(r2, WLibrary.Items[WinLayer_src_image].AsImage, r, False);
  end;

  DIB.Canvas.Brush.Style := bsClear;
  DrawText(DIB.Canvas.Handle, PCHAR(Text), Length(Text), r3,
    DT_VCENTER or DT_SINGLELINE or DT_WORDBREAK or DT_WORD_ELLIPSIS or TextFormat[Alignment]);
  DIB.Canvas.Brush.Style := bsSolid;
  DIB.PremultiplyRect(DIB.Rect);

  Result := DIB;
end;

{ TWinLayerRadioBox }

constructor TWinLayerRadioBox.Create(AOwner: TComponent);
begin
  inherited;
  FChecked := False;
  FButtonState := bsNormal;
end;

destructor TWinLayerRadioBox.Destroy;
begin
  inherited;
end;

procedure TWinLayerRadioBox.SetAlignment(Value: TAlignment);
begin
  if (Value <> taCenter) then
    inherited;
end;

procedure TWinLayerRadioBox.SetChecked(Value: Boolean);
var
  i: Integer;
begin
  if (Value <> FChecked) then
  begin
    FChecked := Value;
    Draw;
    
    if FChecked and Assigned(WinLayerForm) then
    for i:= 0 to WinLayerForm.ControlsCount - 1 do
    if (WinLayerForm.Controls[i] is TWinLayerRadioBox) and
      (WinLayerForm.Controls[i] <> Self) and
      TWinLayerRadioBox(WinLayerForm.Controls[i]).Checked then
    begin
      TWinLayerRadioBox(WinLayerForm.Controls[i]).Checked := False;
      Break;
    end;
  end;
end;

procedure TWinLayerRadioBox.WMLBUTTONUP(var Message: TWMLBUTTONUP);
begin
  inherited;
  if (FButtonState <> bsDisable) and PtInRect(ClientRect, SmallPointToPoint(Message.Pos))
    and not FChecked then SetChecked(True);
end;

function TWinLayerRadioBox.GetDIB: TDIBSection;
const
  ButtonState: array[TWinLayerFormButtonState, Boolean] of String =
  ((WinLayer_src_radiobox_unchecked_normal, WinLayer_src_radiobox_checked_normal),
  (WinLayer_src_radiobox_unchecked_hover, WinLayer_src_radiobox_checked_hover),
  (WinLayer_src_radiobox_unchecked_push, WinLayer_src_radiobox_checked_push),
  (WinLayer_src_radiobox_unchecked_disable, WinLayer_src_radiobox_checked_disable));
var
  DIB: TDIBSection;
  r, r2, r3: TRect;
begin
  DIB := TDIBSection.Create;
  DIB.SetSize(Width, Height, 32);
  DIB.DrawDIB(DIB.Rect, WinLayerForm.SurfaceWindow, BoundsRect);
  DIB.Canvas.Font.Assign(Font);

  with WinLayerForm do
  begin
    r := WLibrary.Items[ButtonState[FButtonState, FChecked]].Rect;

    r2.Top := (DIB.Height - r.Bottom + r.Top) div 2;
    r2.Bottom := r2.Top + r.Bottom - r.Top;
    r3.Top := (DIB.Height - DIB.Canvas.TextHeight('W')) div 2;
    r3.Bottom := r3.Top + DIB.Canvas.TextHeight('W');
    
    case Alignment of
      taLeftJustify:
      begin
        r2.Left := 0;
        r2.Right := r2.Left + r.Right - r.Left;
        r3.Left := r2.Right;
        r3.Right := DIB.Width;
      end;
      taRightJustify:
      begin
        r2.Left := DIB.Width - r.Right + r.Left;
        r2.Right := DIB.Width;
        r3.Left := 0;
        r3.Right := r2.Left;
      end;
    end;

    DIB.DrawPNG(r2, WLibrary.Items[WinLayer_src_image].AsImage, r, False);
  end;

  DIB.Canvas.Brush.Style := bsClear;
  DrawText(DIB.Canvas.Handle, PCHAR(Text), Length(Text), r3,
    DT_VCENTER or DT_SINGLELINE or DT_WORDBREAK or DT_WORD_ELLIPSIS or TextFormat[Alignment]);
  DIB.Canvas.Brush.Style := bsSolid;
  DIB.PremultiplyRect(DIB.Rect);

  Result := DIB;
end;

end.
